package Gtk2::CellRenderer::Base;

use 5.008;
use strict;
use warnings;

use Gtk2;

require DynaLoader;

our @ISA = qw(DynaLoader);


our $VERSION = '0.01';

sub dl_load_flags { 0x01 }

bootstrap Gtk2::CellRenderer::Base $VERSION;

# Preloaded methods go here.

1;
__END__

=head1 NAME

Gtk2::CellRenderer::Base - Base class for custom cell renderers

=head1 SYNOPSIS

  use Gtk2 -init;
  use Gtk2::CellRenderer::Base;

  FIXME

=head1 ABSTRACT

This module allows a Perl developer to derive new cell renderers
for the Gtk2::TreeView.

=head1 DESCRIPTION

The TreeView widget in Gtk+ 2.0 allows for complete customization and 
extensibility through subclassing the cell renderers to fit your needs.
However, Gtk2::CellRenderer does not provide signals for it methods,
and thus because of implementation details requiring that a signal exist
for each virtual method that a language binding wishes to override when
subclassing, it is not possible to derive directly from Gtk2::CellRenderer
in Perl.

Therefore Gtk2::CellRenderer::Base provides a specialized way to derive a
new cell renderer in perl.  Your derived class uses Gtk2::CellRenderer::Base
as a parent class instead of Gtk2::CellRenderer, and all is well.  In the
interest of speed, as cell renderer methods are called I<very> often, this
is implemented with a few shortcuts, so it's not quite like deriving a
normal Glib::Object (which itself is not quite like deriving a normal Perl
object.

=head1 TUTORIAL

FIXME

=head1 SEE ALSO

L<Glib>, L<Gtk2> and the Gtk+ C API reference at
http://developer.gnome.org/doc/API/2.0/gtk/ 

=head1 AUTHOR

muppet E<lt>scott at asofyet dot orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2003 by the gtk2-perl team.

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the 
Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
Boston, MA  02111-1307  USA.

=cut

