#!/usr/bin/perl -w

use Gtk2 -init;

package Mup::CellRendererPopup;

use Gtk2::CellRenderer::Base;

use Glib::Object::Subclass
	'Gtk2::CellRenderer::Base',
	signals => {
		edited => {
			flags => [qw/run-last/],
			#                  path         index
			param_types => [qw/Glib::String Glib::Int/],
		},
	},
	properties => [
		Glib::ParamSpec->boolean ('editable', 'Editable',
		                          'Can i change that?',
					  0, ['readable', 'writable']),
		Glib::ParamSpec->int ('index', 'Index',
		                      'Index of selected list item',
				      0, 65535, 0, [qw/readable writable/]),
		Glib::ParamSpec->boxed ('list', 'List',
		                        'List of possible values',
					'Glib::Scalar',
				        [qw/readable writable/]),
	],
	;

use Data::Dumper;

use constant xpad => 3;
use constant ypad => 2;

sub calc_size {
	my ($cell, $layout) = @_;
	my ($w, $h) = $layout->get_pixel_size;
	return (xpad, ypad, $w + xpad * 2, $h + ypad * 2);
}

sub on_get_size {
	my ($cell, $widget, $area) = @_;
	my $layout = $cell->get_layout ($widget);
	$layout->set_text ($cell->{list}[$cell->{index}]);
	return $cell->calc_size ($layout);
}

sub get_layout {
	my ($cell, $widget) = @_;
	return $widget->create_pango_layout ("");
}

sub on_render {
#	warn "on_render:\n".Dumper(\@_);
	my ($cell, $window, $widget, $background_area, $cell_area, $expose_area, $flags) = @_;
	my $state;
	if (grep {/selected/} @$flags) {
		$state = $widget->has_focus
		       ? 'selected'
		       : 'active';
	} else {
		$state = $widget->state eq 'insensitive'
		       ? 'insensitive'
		       : 'normal';
	}
#	$widget->get_style->paint_box ($window, $widget->state,
#			         'out', $cell_area,
#				 #$widget, "optionmenu",
#				 undef, "optionmenu",
#				 $cell_area->x,
#				 $cell_area->y,
#				 $cell_area->width,
#				 $cell_area->height);

	my $layout = $cell->get_layout ($widget);
	$layout->set_text ($cell->{list}[$cell->{index}]);
	my ($xoff, $yoff, $width, $height) = $cell->calc_size ($layout);
	$widget->get_style->paint_layout ($window,
				$state,
			         1, $cell_area,
				 $widget, "cellrenderertext",
				 $cell_area->x+$xoff,
				 $cell_area->y+$yoff
				 + ($cell_area->height - $height) / 2,
				 $layout);
}

sub menu_pos_func {
	my ($menu, $x, $y, $data) = @_;
	my ($wx, $wy) = $data->[0]->window->get_origin;
	$x = $wx + $data->[1]->x + xpad;
	$y = $wy + $data->[1]->y;
#	# center the menu vertically around the selected item
#	my $active = $menu->get_active;
#	if ($active) {
#		warn "active: $active\n";
#		warn "     y: ".($active->child->get_size_request)."\n";
#		$y -= $active->allocation->y;
#	}
#	foreach my $i ($menu->children) {
#		if ($i == $menu->
#	}
	return ($x, $y, 1);
}

sub editing_done {
	warn Dumper(\@_);
	my ($item, $cell) = @_;
	$cell->signal_emit ('edited', $item->{path}, $item->{index});
}

sub on_start_editing {
	my ($cell, $event, $widget, $path, $background_area, $cell_area, $flags) = @_;
	my $menu = Gtk2::Menu->new;
	my @data = @{ $cell->{list} };
	for (my $i = 0 ; $i < @data ; $i++) {
		my $item = Gtk2::MenuItem->new ($data[$i]);
		$item->show;
		$menu->append ($item);
		$item->{path} = $path;
		$item->{index} = $i;
		$item->{text} = $data[$i];
		$item->signal_connect (activate => \&editing_done, $cell);
	}
	$menu->popup (undef, undef,
	              \&menu_pos_func, [$widget, $cell_area],
#	              sub { ($cell_area->x+xpad, $cell_area->y, 1); }, undef,
	              $event ? $event->button : 0, 0);
	my $item = $menu->get_active;
	$menu->select_item ($item) if $item;
	undef;
}


##########################################################################
# driver code
package main;

$window = Gtk2::Window->new;
$window->set_title ('cell renderer test');
$window->signal_connect (delete_event => sub { Gtk2->main_quit; 0; });

$vbox = Gtk2::VBox->new;
$window->add ($vbox);

$label = Gtk2::Label->new;
$label->set_markup ('<big>F-Words</big>');
$vbox->pack_start ($label, 0, 0, 0);

# create and load the model
$model = Gtk2::ListStore->new ('Glib::String', 'Glib::Scalar', 'Glib::Int');
foreach ([ 'foo',        [qw/foo bar baz/]],
         [ 'fluffy',     [qw/muffy tuffy buffy willow/]],
         [ 'flurble',    [qw/murble swurble curble/]],
         [ 'frob',       [qw/blob clob plob mob rob gob glob wob dob/]],
         [ 'frobnitz',   [qw/fronbination that's sweepin' the nation/]],
         [ 'ftang',      [qw/quisinart/]],
         [ 'fire truck', [qw/red white green yellow polka-dot/]]) {
	my $iter = $model->append;
	$model->set ($iter, 0, $_->[0], 1, $_->[1], 2, rand (@{$_->[1]}));
}


# now a view
$treeview = Gtk2::TreeView->new ($model);

# regular text column for column 0, the string
$renderer = Gtk2::CellRendererText->new;
$renderer->set (editable => 1);;
$column = Gtk2::TreeViewColumn->new_with_attributes ('something', $renderer,
                                                     text => 0);
$treeview->append_column ($column);

# text for col[1]
$renderer = Gtk2::CellRendererText->new;
$column = Gtk2::TreeViewColumn->new_with_attributes ('list', $renderer,);
$column->set_cell_data_func ($renderer, sub {
		my ($tree_column, $cell, $model, $iter) = @_;
		my ($info) = $model->get ($iter, 1);
		$cell->set (text => "[".join(", ", @$info)."]");
	});
$treeview->append_column ($column);

# text for col[2]
$renderer = Gtk2::CellRendererText->new;
$column = Gtk2::TreeViewColumn->new_with_attributes ('index', $renderer,
                                                     text => 2);
$treeview->append_column ($column);

# custom for cols 1 and 2
$renderer = Mup::CellRendererPopup->new;
$renderer->set (mode => 'editable');
$column = Gtk2::TreeViewColumn->new_with_attributes ('selector', $renderer,
                                                     list => 1,
						     'index' => 2,
						     );
$renderer->signal_connect (edited => sub {
		my ($cell, $text_path, $new_index, $model) = @_;
		my $path = Gtk2::TreePath->new_from_string ($text_path);
		my $iter = $model->get_iter ($path);
		$model->set ($iter, 2, $new_index);
	}, $model);
$treeview->append_column ($column);

#$scrolled = Gtk2::ScrolledWindow->new;
#$scrolled->set_policy ('automatic', 'automatic');
#$scrolled->add ($treeview);
#$vbox->pack_start ($scrolled, 1, 1, 0);
$vbox->pack_start ($treeview, 1, 1, 0);

$window->show_all;

Gtk2->main;
